UNIT Dynamics;

INTERFACE

USES Global,OwlGraph,OwlAsm,Utility,RuleSet,Genes,Linlib,Printer,mapbind;

PROCEDURE InitializePop(VAR OwlRec : Population;VAR Error : Boolean);
PROCEDURE PairToFemale(    X,Y      : Word;
                       VAR Pop  : Population;
                           PairLoc  : Word);
PROCEDURE PairToMale  (    X,Y      : Word;
                       VAR Pop  : Population;
                           PairLoc  : Word);
PROCEDURE MakePair  (X,Y       : Word;
                     VAR Pop   : Population;
                     MLoc,FLoc : Word);
PROCEDURE Fledge(    X,Y,NumMale,NumFemale,PairLoc : Word;
                     PairType                      : Byte;
                 VAR Pop                           : Population);
PROCEDURE KillOwlOrPair(X,Y,Loc : Word;VAR Pop : Population;OwlType : Byte);
PROCEDURE InitializeOwlPop(VAR Pop : Population);
PROCEDURE FemaleMove(    X,Y,Loc    : Word;
                     VAR Pop        : Population;
                         BorderType : Byte);
PROCEDURE MaleMove(    X,Y,Loc    : Word;
                   VAR Pop        : Population;
                       BorderType : Byte);
Procedure DoDyamicMap(    X,Y,Year : Word;
                      VAR Map      : HabitatArr;
                      VAR Pop      : Population;
                      VAR FileName : String);
FUNCTION SearchForOwls(    X,Y        : Word;
                        VAR Pop       : Population;
                            Class     : Byte;
                        VAR Age       : Byte):Byte;

IMPLEMENTATION
CONST
     StatusWord : Word = 0;

(****Assembly procedures to search records and collapse arrays*****)

PROCEDURE Collapse(N,Position,Size : Word; VAR Arr); EXTERNAL;
{$L ClpxArr}
FUNCTION Search(N,X,Y,Size : Word; VAR Arr) : Word; EXTERNAL;
FUNCTION SearchForSettled(N,X,Y,Size : Word; VAR Arr) : Word; EXTERNAL;
{$L Srch}
PROCEDURE DecrementIndex(n,Pos : Word;VAR Arr);EXTERNAL;
{$L dec}

VAR
   HabitatPtr                 : Pointer;

FUNCTION GetDistance(Xstart,YStart,XStop,YStop : Integer):Single;
         BEGIN
             GetDistance := Sqrt(Sqr(LongInt(XStart-XStop)) +
                                 Sqr(LongInt(YStart-Ystop)));
END;
PROCEDURE CollapseIndex(N,Pos : Word; VAR Arr : IndexType);
          VAR
             Ct     : Word;
             Holder : Word;
          BEGIN
               Holder := Arr[Pos];
               Collapse(N,Pos,2,Arr);
               DecrementIndex(Pred(N),Holder,Arr);
END;

PROCEDURE CollapseM(N,Position : Integer; VAR Pop : Population);
          VAR
             Ct : Word;
          BEGIN
               WITH Pop DO BEGIN
                         Collapse(N,Position,SizeOf(SingleMale),MalePop^);
                         CollapseIndex(N,Position,MIndx);
                         Dec(MaleCtr);
               END;
END;
PROCEDURE CollapseF(N,Position : Integer; VAR Pop : Population);
          VAR
             Ct : Word;
          BEGIN
               WITH Pop DO BEGIN
                         Collapse(N,Position,SizeOf(SingleFemale),FemalePop^);
                         CollapseIndex(N,Position,FIndx);
                         Dec(FemaleCtr);
               END;
END;


PROCEDURE CollapseP(N,Position : Integer;VAR Pop : Population);
          VAR
             Ct : Word;
          BEGIN
               WITH Pop DO BEGIN
                         Collapse(N,Position,SizeOf(Pair),PairPop^);
                         Dec(PairCtr);
                    END;
END;
PROCEDURE CheckLimit(Size : Word);
          BEGIN
               IF Size > MaxOwls THEN BEGIN
                  IF GraphOn THEN
                     RestoreTextMode;
                     FatalErrors(3);
                  END;
END;
PROCEDURE InitializePop(VAR OwlRec : Population;VAR Error : Boolean);
          VAR
             Ct1 : Word;
          BEGIN
               Error := FALSE;
               IF MaxAvail > (SizeOf(MaleArr) + SizeOf(FemaleArr) +
                              SizeOf(PairArr)) + (Years+1)*sizeOf(CtrType)
               THEN BEGIN
                    WITH OwlRec DO BEGIN
                         New(MalePop);
                         New(FemalePop);
                         New(PairPop);
                          FOR Ct1 := 0 TO Years DO BEGIN
                              New(Totals[Ct1]);
                              ZeroFill(Totals[Ct1]^,sizeof(Totals[ct1]^));
                          END;
                         MaleCtr      := 0;
                         FemaleCtr   := 0;
                         PairCtr     := 0;
                    END;
               END
               ELSE BEGIN
                    FatalErrors(2);
               END;
  END;
FUNCTION WrapAroundX(X : Integer) : Integer;
        BEGIN
              IF X < 1 THEN
                 X := HabitatWidth
              ELSE
                  IF X > HabitatWidth THEN
                     X := 1;
                 WrapAroundX := X;
END;
FUNCTION WrapAroundY(Y : Integer) : Integer;
         BEGIN
              IF Y < 1 THEN
                 Y := HabitatLength
              ELSE
                  IF Y > HabitatLength THEN
                     Y := 1;
                 WrapAroundY := Y;
END;

FUNCTION Acceptable(MapId : Word): Boolean;
         VAR
            Class,Owl : Byte;
         BEGIN
              Class := GetClass(MapId);
              Owl   := GetOwl(MapId);
              Acceptable := ((LandClassOK[Class] > Rand) AND ((Owl <= SFemale)));
END;

{If Males Are On Acceptable sites, then they may become Settled}

PROCEDURE TwoOrMoreOwls(X,Y : Word;NewType : Byte; Loc:Word;
                      VAR Pop:Population);
          CONST
               RestrictiveRanking : Array[0..4] OF Byte =
                                  (0,2,1,4,3);

          VAR
             OldType : Byte;
          BEGIN
                  IF RestrictiveRanking[NewType] >
                     RestrictiveRanking[GetOwl(Habitat[Y]^[X])] THEN  BEGIN
                        IF (NewType = SMale) AND Acceptable(Habitat[Y]^[X]) THEN BEGIN
                           NewType := SettledMale;
                           Pop.MalePop^[Loc].Settled := TRUE;
                        END;
                        AddOwl(NewType,Habitat[Y]^[X]);
                  END;
END;
FUNCTION SearchForOwls(    X,Y        : Word;
                        VAR Pop       : Population;
                            Class     : Byte;
                        VAR Age       : Byte):Byte;
VAR
     Loc : Word;
     M : SingleMale;
     F : SingleFemale;
BEGIN
     Loc := 0;
     WITH Pop DO BEGIN
      CASE Class OF
 OwlPair,
 SettledMale,
 SMale
                  :   BEGIN
                        Loc :=  Search(MaleCtr,X,Y,SizeOf(SingleMale),MalePop^);
                        IF Loc <> 0 THEN BEGIN
                          M := Pop.MalePop^[Loc];
                          Age := M.Age;
                          SearchForOwls := SMale;
                        END
                        ELSE BEGIN
                          Loc := Search(FemaleCtr,X,Y,SizeOf(SingleFemale),FemalePop^);
                          IF Loc  <> 0 THEN BEGIN
                             F := Pop.FemalePop^[Loc];
                             Age := F.Age;
                             SearchForOwls := SFemale
                          END;
                        END;
                        IF (Loc = 0) THEN
                            SearchForOwls := 0;
                   END;
         SFemale : BEGIN
                        Loc := Search(FemaleCtr,X,Y,SizeOf(SingleFemale),FemalePop^);
                        IF Loc <> 0
                           THEN BEGIN
                              SearchForOwls := SFemale;
                              Age := Pop.FemalePop^[Loc].Age;
                           END
                        ELSE
                            SearchForOwls := 0;
                   END;
      END;
     END;
END;
PROCEDURE ReturnOwlType(X,Y : Word; OwlType : Byte;VAR Pop : Population;
                          VAR Age : Byte; VAR ChangeBird : Boolean);
          VAR
             Owl, Owl2 : Byte;
          BEGIN
               Owl := GetOwl(Habitat[Y]^[X]);
                 {If the owl is not a fixed entity and the owl to be moved is
                  the defining type then we need to see what is left}

                 IF (Owl = OwlType) THEN BEGIN
                    Owl := SearchForOwls(X,Y,Pop,OwlType, Age);
                    AddOwl(Owl,Habitat[Y]^[X]);
                    IF Owl = 0 THEN
                       NoMultiple(Habitat[Y]^[X]);
                 END
                 ELSE BEGIN
                    ChangeBird := FALSE;
                    Owl2 := SearchForOwls(X,Y,Pop, Owl,Age);
                    IF (Owl2 = 0) THEN
                       NoMultiple(Habitat[Y]^[X]);

                 END;
END;

{Procedure transfer pair data assumes a particular format for the owl
 records.  If these record formats are changed, then this procedure will
 need to be re-written.  TransferPairData moves the location and alleles
 from a pair to an owl. It also fills in the direction as settled and,
 in the case of a male, sets Settled to TRUE}

Procedure TransferPairData(VAR FromPair,ToOwl;Male : Boolean); assembler;
asm
   mov dx,ds
   lds si,fromPair
   les di,ToOwl
   mov cl, byte ptr Male
   cld
   movsw     {move row,col}
   movsw
   mov al,7  {move in direction = settled}
   stosb
   or cl,cl
   jz @Female
      mov al,1    {male, make it settled}
      stosb
      Jmp @finis
   @female:
      add si,4    {move to second allele pair}
   @finis:
      movsw       {move in the alleles}
      movsw
   mov ds,dx
end;

PROCEDURE PairToFemale     (X,Y      : Word;
                            VAR Pop  : Population;
                            PairLoc  : Word);
          VAR
             F        : SingleFemale;
             TempPair : Pair;
             NewBird  : Byte;
             Age      : Byte;
          BEGIN
               NewBird := SFemale;
               WITH Pop DO BEGIN
                       TempPair := PairPop^[PairLoc];
                        CollapseP(PairCtr,PairLoc,Pop);                      
                        TransferPairData(TempPair,F,FALSE);
                        F.Age        := TempPair.FemaleAge;
                        F.StartX     := 65535;
                        F.StartY     := 65535;
                        F.EverPaired := TRUE;
                        Inc(FemaleCtr);
                        CheckLimit(FemaleCtr);
                        FastMove(F,FemalePop^[FemaleCtr],
                                   SizeOf(SingleFemale));
                        
               end;
               IF IsMultiple(Habitat[Y]^[X]) THEN 
                  AddOwl(SearchForOwls(X,Y,Pop,OwlPair,Age),Habitat[Y]^[X])
               ELSE  BEGIN
                   Age := F.Age;
                   AddOwl(NewBird,Habitat[Y]^[X]);
               END;
               IF GraphOn THEN
                  GraphicChangeBird(X,y);
END;
PROCEDURE PairToMale     (  X,Y      : Word;
                            VAR Pop  : Population;
                            PairLoc  : Word);
          VAR
             M        : SingleMale;
             TempPair : Pair;
             NewBird  : Byte;

          BEGIN
               NewBird := SettledMale;
               WITH Pop DO BEGIN
                       TempPair := PairPop^[PairLoc];
                       CollapseP(PairCtr,PairLoc,Pop);
                       TransferPairData(TempPair,M,TRUE);
                       M.Age        := TempPair.MaleAge;
                       M.StartX     := 65535;
                       M.StartY     := 65535;
                       M.EverPaired := TRUE;
                       Inc(MaleCtr);
                       CheckLimit(MaleCtr);
                       FastMove(M,MalePop^[MaleCtr],
                                   SizeOf(SingleMale));
               end;
               AddOwl(NewBird,Habitat[Y]^[X]);
               IF GraphOn THEN
                  GraphicChangeBird(X,Y);
END;
Procedure SplitPair(x,y : Word;VAR Pop : Population);
          VAR
              SearchLoc : Word;
              Ctr       : Word;
              F         : SingleFemale;
              M         : SingleMale;
              P         : Pair;
              NewBird   : Byte;
              AGe       : Byte;
          begin
               NewBird := SMale;
               With pop do begin
                       SearchLoc := Search(PairCtr,x,y,SizeOf(Pair),
                                      PairPop^);
                       if SearchLoc = 0 then begin
                          WriteLn('Error');
                          Halt;
                       end;
                       P := PairPop^[SearchLoc];
                       CollapseP(PairCtr,SearchLoc,Pop);
                       TransferPairData(P,F,FALSE);
                       F.Age        := P.FemaleAge;
                       F.StartX     := 65535;
                       F.StartY     := 65535;
                       Inc(FemaleCtr);
                       CheckLimit(FemaleCtr);
                       FastMove(F,FemalePop^[FemaleCtr],
                                   SizeOf(SingleFemale));
                       TransferPairData(P,M,TRUE);
                       M.Age        := P.MaleAge;
                       M.StartX     := 65535;
                       M.StartY     := 65535;
                       M.Settled    := FALSE;
                       Inc(MaleCtr);
                       CheckLimit(MaleCtr);                       
                       FastMove(M,MalePop^[MaleCtr],
                                   SizeOf(SingleMale));
               end;
               AddOwl(SearchForOwls(X,Y,Pop,OwlPair,Age),Habitat[Y]^[X]);
               MakeMultiple(Habitat[Y]^[X]);
end;

PROCEDURE MakePair  (X,Y       : Word;
                     VAR Pop   : Population;
                     MLoc,FLoc : Word);
VAR
   P           : Pair;
   M           : SingleMale;
   F           : SingleFemale;
   
BEGIN
     WITH Pop DO BEGIN
       F := FemalePop^[FLoc];
       M := MalePop^[MLoc];
       IF (F.EverPaired = FALSE) THEN BEGIN
          Inc(PairDistance.N);
          PairDistance.SumDist := PairDistance.SumDist +
                              GetDistance(F.StartX,X,F.StartY,Y);
       END;
          IF (M.EverPaired = FALSE) THEN BEGIN
          Inc(PairDistance.N);
          PairDistance.SumDist := PairDistance.SumDist +
                            GetDistance(M.StartX,X, M.StartY,Y);
       END;
       CollapseM(MaleCtr,MLoc,Pop);
       CollapseF(FemaleCtr,FLoc,Pop);
       Inc(PairCtr);
       CheckLimit(PairCtr);
       P.Row     := Y;
       P.Column  := X;
       P.Allele1 := M.Allele1;
       P.Allele2 := M.Allele2;
       P.Allele3 := F.Allele1;
       P.Allele4 := F.Allele2;
       P.MaleAge   := M.Age;
       P.FemaleAge := F.Age;
       FastMove(P,PairPop^[PairCtr],SizeOf(Pair));
       AddOwl(OwlPair,Habitat[Y]^[X]);
     END;
        
END;
FUNCTION FindPair(X,Y : Word;VAR OwlPop : Population) : Word;
          BEGIN
               WITH OwlPop DO BEGIN
                 FindPair := SearchForSettled(MaleCtr,X,Y,SizeOf(SingleMale),MalePop^)
               END;
END;

Procedure DeathDist(VAR D   : Distance;
                    VAR Pop : Population;
                        Loc : Word;
                        Male: Boolean);
          VAR
             M : SingleMale;
             F : SingleFemale;
          begin
               With D do begin
                    if Male then begin
                            M := Pop.MalePop^[Loc];
                            if M.StartX <> 65535 then begin
                               Inc(N);
                               SumDist :=  SumDist +
                                           GetDistance(M.StartX,
                                                       M.Column,
                                                       M.StartY,
                                                       M.Row);
                            end;
                    end
                    else begin
                         F := Pop.FemalePop^[Loc];
                            if F.StartX <> 65535 then begin
                               Inc(N);
                               SumDist :=  SumDist +
                                           GetDistance(F.StartX,
                                                       F.Column,
                                                       F.StartY,
                                                       F.Row);
                            end;
                    end;
               end;
end;
PROCEDURE KillOwlOrPair(X,Y,Loc : Word;VAR Pop : Population;OwlType : Byte);
VAR
   ChangeBird : Boolean;
   Age        : Byte;
          BEGIN
               ChangeBird := TRUE;
               CASE OwlType OF
       SettledMale,SMale   : BEGIN
                                   DeathDist(DeathDistance,Pop,Loc,True);
                                   CollapseM(Pop.MaleCtr,Loc,Pop);
                              END;
                    SFemale : BEGIN
                                   DeathDist(DeathDistance,Pop,Loc,False);
                                   CollapseF(Pop.FemaleCtr,Loc,Pop);
                              END;
                  
                    OwlPair   : CollapseP(Pop.PairCtr,Loc,Pop);
               END;
               IF IsMultiple(Habitat[Y]^[X]) THEN
                  ReturnOwlType(X,Y,OwlType,Pop, Age, ChangeBird)
               ELSE
                   KillOwl(Habitat[Y]^[X]);
               IF GraphOn AND ChangeBird THEN
                  GraphicChangeBird(X,Y);
END;
PROCEDURE Fledge(    X,Y,NumMale,NumFemale,PairLoc : Word;
                     PairType                      : Byte;
                 VAR Pop                           : Population);
          VAR
             Ct : Word;
             M  :  SingleMale;
             F  : SingleFemale;
          BEGIN
               IF (NumMale + NumFemale) = 0 THEN Exit;
               WITH Pop DO BEGIN
                    M.Age        := 0;
                    M.Row        := Y;
                    M.Column     := X;
                    M.Direction  := 7;
                    M.Settled    := FALSE;
                    M.StartX     := X;
                    M.StartY     := Y;
                    M.EverPaired := FALSE;
                    FOR Ct := 1 TO NumMale DO BEGIN
                        Inc(MaleCtr);
                        CheckLimit(MaleCtr);
                        FastMove(M,MalePop^[MaleCtr],SizeOf(SingleMale));
                        MakeYoungGenes(SMale,PairType,PairLoc,MaleCtr,Pop);
                    END;
                    F.Age        := 0;
                    F.Row        := Y;
                    F.Column     := X;
                    F.Direction  := 7;
                    F.StartX     := X;
                    F.StartY     := Y;
                    F.EverPaired := FALSE;
                    FOR Ct := 1 TO NumFemale DO BEGIN
                        Inc(FemaleCtr);
                        CheckLimit(FemaleCtr);
                        FemalePop^[FemaleCtr] := F;
                        FastMove(F,FemalePop^[FemaleCtr],SizeOf(SingleFemale));
                        MakeYoungGenes(SFemale,PairType,PairLoc,FemaleCtr,Pop);
                    END;
                    MakeMultiple(Habitat[Y]^[X]);
               END;
END;

PROCEDURE InitializeOwlPop(VAR Pop : Population);
          VAR
             Ct1,Ct2 : Word;
             Owl     : Byte;
             M       : SingleMale;
             F       : SingleFemale;
             P       : Pair;
     BEGIN
          ZeroFill(PairDistance,SizeOf(PairDistance));
          ZeroFill(DeathDistance,SizeOf(DeathDistance));
          WITH Pop DO BEGIN
               FOR Ct1 := 1 TO HabitatLength DO BEGIN
                   FOR Ct2 := 1 TO HabitatWidth DO BEGIN
                       ZeroFill(M,SizeOf(M));
                       ZeroFill(F,SizeOf(F));
                       NoMultiple(Habitat[Ct1]^[Ct2]);
                       Owl := GetOwl(Habitat[Ct1]^[Ct2]);
                       IF Owl <> 0 THEN BEGIN
                         CASE Owl OF
                            SMale: BEGIN
                                         M.Row        := Ct1;
                                         M.Column     := Ct2;
                                         M.Direction  := 7;
                                         M.StartX     := Ct2;
                                         M.StartY     := Ct1;
                                         M.Age        := MaxStage;
                                         M.EverPaired := FALSE;
                                         IF Acceptable(Habitat[Ct1]^[Ct2])
                                            THEN BEGIN
                                                 M.Settled := TRUE;
                                                 AddOwl(SettledMale,
                                                        Habitat[Ct1]^[Ct2]);
                                         END;
                                         Inc(MaleCtr);
                                         CheckLimit(MaleCtr);
                                         MalePop^[MaleCtr] := M;
                                    END;
                          SFemale  : BEGIN
                                         F.Row        := Ct1;
                                         F.Column     := Ct2;
                                         F.Direction  := 7;
                                         F.StartX     := Ct2;
                                         F.StartY     := Ct1;
                                         F.Age        := MaxStage;
                                         F.EverPaired := FALSE;
                                         Inc(FemaleCtr);
                                         CheckLimit(FemaleCtr);
                                         FemalePop^[FemaleCtr] := F;
                                    END;
                            OwlPair : BEGIN
                                         P.Row        := Ct1;
                                         P.Column     := Ct2;
                                         P.MaleAge    := MaxStage;
                                         P.FemaleAge  := MaxStage;
                                         Inc(PairCtr);
                                         CheckLimit(PairCtr);
                                         PairPop^[PairCtr] := P;
                                    END;
                         END;
                       END;
                   END;
               END;
               FillWithGenes(Pop);
          END;
END;

{MoveOwl updates the arrays and also dictates the map configuration -
 it is assumed that, at this point, the move is valid : Ok land class
 and no territorial disputes
}

PROCEDURE MoveOwl(    X,Y,NewX,NewY,Loc : Word;
                      Direction         : Byte;
                  VAR Pop               : Population;
                      OwlType           : Byte);
          VAR
             MapId1  : Byte;
             MapId2  : Byte;
             Owl     : Byte;
             MaleLoc : Word;
             Age1,
             Age2    : Byte;
             ChangeBird : Boolean;
             Test : Boolean;
      BEGIN
         {
           If (X > Habitatwidth) or (Y > HabitatLength) or
              (Y < 1) or (X < 1) then begin
                 WriteLn('Error - Bad X,Y Passed to MoveOwl');
                 ReadLn;
                 Halt;
            end;

            If (NewX > Habitatwidth) or (NewY > HabitatLength) or
              (NewY < 1) or (NewX < 1) then begin
                 WriteLn('Error - Bad NewX,NewY Passed to MoveOwl');
                 ReadLn;
                 Halt;
            end;
         }
           ChangeBird := TRUE;
           WITH Pop DO BEGIN
                CASE OwlType OF
                     SMale : BEGIN
                                  MalePop^[Loc].Row        := NewY;
                                  MalePop^[Loc].Column     := NewX;
                                  MalePop^[Loc].Direction  := Direction;
                             END;
                   SFemale : BEGIN
                                  FemalePop^[Loc].Row        := NewY;
                                  FemalePop^[Loc].Column     := NewX;
                                  FemalePop^[Loc].Direction  := Direction;
                             END;
                   END;

               MapId1 := Habitat[Y]^[X];
               MapId2 := Habitat[NewY]^[NewX];
               Test := IsMultiple(MapID1);
               IF IsMultiple(MapId1) THEN BEGIN  {more than 1 is present}
                  ReturnOwlType(X,Y,OwlType,Pop,Age1, ChangeBird);
               END
               ELSE
                   KillOwl(Habitat[Y]^[X]);

               {done with the details of the old location, now on to
                the new one}

               Owl := GetOwl(MapId2);
               CASE OwlType OF
                    SFemale : BEGIN
                               IF Owl <> 0 THEN BEGIN
                                 IF Owl >= SettledMale THEN BEGIN
                                    MaleLoc := FindPair(NewX,NewY, Pop);
                                    IF FemalePop^[Loc].Age <= MalePop^[MaleLoc].Age THEN
                                       Age2 := FemalePop^[Loc].Age
                                    ELSE
                                        Age2 := MalePop^[MaleLoc].Age;
                                    MakePair(NewX,NewY,Pop,MaleLoc,Loc);
                                   
                                 END
                                 ELSE IF Owl <> 0 THEN BEGIN
                                      MakeMultiple(Habitat[NewY]^[NewX]);
                                      TwoOrMoreOwls(NewX,NewY,OwlType,Loc,Pop);
                                      IF (Owl = OwlPair) THEN BEGIN
                                        Loc := Search(PairCtr,NewX,NewY,
                                                SizeOf(Pair),PairPop^);
                                        IF (PairPop^[Loc].MaleAge <=
                                            PairPop^[Loc].FemaleAge) THEN
                                               Age2 := PairPop^[Loc].MaleAge
                                        ELSE Age2 := PairPop^[Loc].FemaleAge;
                                      END
                                      ELSE IF (Owl = SMale) THEN BEGIN
                                         Loc := Search(MaleCtr,NewX,NewY,
                                            SizeOf(singleMale),MalePop^);
                                         Age2 := MalePop^[Loc].Age;
                                      END
                                      ELSE IF (Owl = SFemale) THEN BEGIN
                                         Loc := Search(FemaleCtr,NewX,NewY,
                                             SizeOf(SingleFemale), FemalePop^);
                                         Age2 := FemalePop^[Loc].AGe;
                                      END;
                                     

                                 END
                               END
                               ELSE  BEGIN
                                   AddOwl(OwlType,Habitat[NewY]^[NewX]);
                                   Age2 := Pop.FemalePop^[Loc].Age;
                                END;
                              END;
                    SMale   : BEGIN
                                IF Owl <> 0 THEN BEGIN
                                   MakeMultiple(Habitat[NewY]^[NewX]);
                                   TwoOrMoreOwls(NewX,NewY,OwlType,Loc,Pop);
                                   IF (Owl = OwlPair) THEN BEGIN
                                       Loc := Search(PairCtr,NewX,NewY,
                                                SizeOf(Pair),PairPop^);
                                        IF (PairPop^[Loc].MaleAge <=
                                            PairPop^[Loc].FemaleAge) THEN
                                               Age2 := PairPop^[Loc].MaleAge
                                        ELSE Age2 := PairPop^[Loc].FemaleAge;
                                   END
                                   ELSE BEGIN
                                   Loc := Search(Pop.Malectr,Newx,NewY,
                                        SizeOf(singleMale),Pop.MalePop^);
                                   Age2 := Pop.MalePop^[Loc].Age;
                                   END;
                                END
                                ELSE
                                   IF Acceptable(MapId2) THEN BEGIN
                                     AddOwl(SettledMale,Habitat[NewY]^[NewX]);
                                     MalePop^[Loc].Settled := TRUE;
                                     Age2 := MalePop^[Loc].Age;
                                   END
                                   ELSE BEGIN
                                      Age2 := MalePop^[Loc].Age;
                                      AddOwl(SMale,Habitat[NewY]^[NewX]);
                                   END;
                               END;
               END;
           END;
           IF GraphOn THEN BEGIN
              IF ChangeBird THEN
                 GraphicChangeBird(X,Y);
              GraphicChangeBird(NewX,NewY);
           END;
END;
PROCEDURE GetCoordinates(VAR X,Y           : Word;  {Change to inc,dec}
                             Direction     : Byte;
                             BoundaryType  : Byte;
                         VAR OffMap        : Boolean);
           BEGIN
                OffMap := FALSE;
                IF Odd(Y) THEN BEGIN
                   CASE Direction OF
                        1 :  BEGIN
                                  Dec(Y);
                                  Inc(X);
                             END;
                        2 :  Inc(X);
                        3 :  BEGIN
                                  Inc(Y);
                                  Inc(X);
                             END;
                        4 :  Inc(Y);
                        5 :  Dec(X);
                        6 :  Dec(Y);
                   END;
                END
                ELSE BEGIN
                     CASE Direction OF
                          1 :  Dec(Y);
                          2 :  Inc(X);
                          3 :  Inc(Y);
                          4 :  BEGIN
                                    Inc(Y);
                                    Dec(X);
                               END;
                          5 :  Dec(X);
                          6 :  BEGIN
                                    Dec(Y);
                                    Dec(X);
                               END;
                     END;
                END;
                IF (BoundaryType = AbsorbingBorder) THEN BEGIN
                   IF (X < 1) OR (X > HabitatWidth) OR
                      (Y < 1) OR (Y > HabitatLength)
                   THEN
                       OffMap := TRUE;
                END
                ELSE IF BoundaryType = ContinuousBorder THEN BEGIN
                     Y := WrapAroundY(Y);
                     X := WrapAroundX(X);
                END;
END;
PROCEDURE GetSurroundings(    X,Y          : Word;
                          VAR Arr          : LandTypeArr;
                              BoundaryType : Byte);
          VAR
             XP1,XM1,YP1,YM1 : Word;
             Ct              : Word;
          BEGIN
               IF (X > 1) AND (X < HabitatWidth) AND    {not at an edge}
                  (Y > 1) AND (Y < HabitatLength)
                     THEN BEGIN
                          Arr[7] := Habitat[Y]^[X];
                          IF Odd(Y) THEN BEGIN
                             Arr[1] := Habitat[Y-1]^[X+1];
                             Arr[2] := Habitat[Y]^[X+1];
                             Arr[3] := Habitat[Y+1]^[X+1];
                             Arr[4] := Habitat[Y+1]^[X];
                             Arr[5] := Habitat[Y]^[X-1];
                             Arr[6] := Habitat[Y-1]^[X];
                          END
                          ELSE BEGIN
                               Arr[1] := Habitat[Y-1]^[X];
                               Arr[2] := Habitat[Y]^[X+1];
                               Arr[3] := Habitat[Y+1]^[X];
                               Arr[4] := Habitat[Y+1]^[X-1];
                               Arr[5] := Habitat[Y]^[X-1];
                               Arr[6] := Habitat[Y-1]^[X-1];
                          END;
                     END ELSE BEGIN
                      IF (Boundarytype = AbsorbingBorder) OR
                         (Boundarytype = ReflectingBorder) THEN BEGIN
                         IF BoundaryType = AbsorbingBorder THEN
                         {just get the class}
                            FillType(GetClass(Habitat[Y]^[X]) SHL 4,Arr)
                         ELSE
                              FillType(ReflectingType,Arr);
                         Arr[7] := Habitat[Y]^[X];
                         XP1 := X + 1;
                         XM1 := X - 1;
                         YP1 := Y + 1;
                         YM1 := Y - 1;
                         IF Odd(Y) THEN BEGIN
                           IF (XP1 <= HabitatWidth) THEN BEGIN
                              Arr[2] := Habitat[Y]^[XP1];
                            IF (YM1 >= 1) THEN
                               Arr[1] := Habitat[YM1]^[XP1];
                           END;
                           IF (YP1 <= HabitatLength) THEN BEGIN
                             Arr[4] := Habitat[YP1]^[X];
                             IF (XP1 <= HabitatWidth) THEN
                                Arr[3] := Habitat[YP1]^[XP1];
                           END;
                           IF (YM1 >= 1) THEN
                              Arr[6] := Habitat[YM1]^[X];
                           IF (XM1 >= 1) THEN
                              Arr[5] := Habitat[Y]^[XM1];
                          END
                          ELSE BEGIN
                               IF (YM1 >= 1) THEN
                                  Arr[1] := Habitat[YM1]^[X];
                               IF (XP1 <= HabitatWidth) THEN
                                  Arr[2] := Habitat[Y]^[XP1];
                             IF (YP1 <= HabitatLength) THEN BEGIN
                                Arr[3] := Habitat[YP1]^[X];
                                IF (XM1 >= 1) THEN
                                   Arr[4] := Habitat[YP1]^[XM1];
                             END;
                              IF (XM1 >= 1) THEN BEGIN
                                 Arr[5] := Habitat[Y]^[XM1];
                                 IF (YM1 >= 1) THEN
                                    Arr[6] := Habitat[YM1]^[XM1];
                              END;
                          END;
                      END
{WrapAround}          ELSE BEGIN
                           Arr[7] := Habitat[Y]^[X];
                           IF Odd(Y) THEN BEGIN
                             Arr[1] := Habitat[WrapAroundY(Y-1)]^[WrapAroundX(X+1)];
                             Arr[2] := Habitat[Y]^[WrapAroundX(X+1)];
                             Arr[3] := Habitat[WrapAroundY(Y+1)]^[WrapAroundX(X+1)];
                             Arr[4] := Habitat[WrapAroundY(Y+1)]^[X];
                             Arr[5] := Habitat[Y]^[WrapAroundX(X-1)];
                             Arr[6] := Habitat[WrapAroundY(Y-1)]^[X];
                          END
                          ELSE BEGIN
                               Arr[1] := Habitat[WrapAroundY(Y-1)]^[X];
                               Arr[2] := Habitat[Y]^[WrapAroundX(X+1)];
                               Arr[3] := Habitat[WrapAroundY(Y+1)]^[X];
                               Arr[4] := Habitat[WrapAroundY(Y+1)]^[WrapAroundX(X-1)];
                               Arr[5] := Habitat[Y]^[WrapAroundX(X-1)];
                               Arr[6] := Habitat[WrapAroundY(Y-1)]^[WrapAroundX(X-1)];
                          END;
                      END;
                     END;
END;
FUNCTION GetDirection(Loc : Word;VAR Pop : Population;OwlType : Byte):Byte;
         BEGIN
              WITH Pop DO BEGIN
                   CASE OwlType OF
                        SMale : GetDirection := MalePop^[Loc].Direction;
                      SFemale : GetDirection := FemalePop^[Loc].Direction;
                   END;
              END;
END;

{ChooseMove makes a weighted random decision on which direction to move
 weighting factors are : land classification, presence of pairs or settled
 males (just for males), and previous direction}

{two support files}

FUNCTION GetChoice(Rand : Single;VAR Arr) : Byte;EXTERNAL;
{$L GetMove}

FUNCTION ChooseMove(VAR Types   : LandTypeArr;
                        OwlDir,
                        OwlType : Byte): Byte;
         TYPE
             SingArr = Array[1..7] OF Single;
         CONST
              HoldArr : SingArr = (1/7,1/7,1/7,1/7,1/7,1/7,1/7);

         VAR
            Arr  : SingArr;
            Ct   : Byte;
         BEGIN
              FastMove(HoldArr,Arr,SizeOf(SingArr));
              IF (OwlType = SMale)  THEN
                      FOR Ct := 1 TO 7 DO
                          IF GetOwl(Types[Ct]) = OwlPair THEN {fixed}
                              Arr[Ct] := Arr[Ct] * PairAversion;
              FOR Ct := 1 TO 7 DO
                  Arr[Ct] := Arr[Ct] * Aversion[GetClass(Types[Ct])];
              Arr[OwlDir] := Arr[OwlDir] * DirectionalWeight;
              Arr[7] := Arr[7] * WanderLust[GetClass(Types[7])];
              ChooseMove := GetChoice(Rand,Arr);
END;

PROCEDURE NoMove(Loc : Word;VAR Pop : Population;OwlType : Byte);
         BEGIN
              WITH Pop DO BEGIN
                   CASE OwlType OF
                        SMale :  MalePop^[Loc].Direction := 7;
                      SFemale :  FemalePop^[Loc].Direction := 7;
                   END;
              END;
END;


{Females must first check for mates - this gets priority over
 all other considerations.  Obligately, she checks the cell that
 she is presently in, then randomly checks the adjacent cells.  If mates
 are not found, then she checks the landtypes for preference, and
 based on her previous position, moves}

PROCEDURE FemaleMove(    X,Y,Loc    : Word;
                     VAR Pop        : Population;
                         BorderType : Byte);
          VAR
             OwlDir    : Byte;
             Types     : LandTypeArr;
             Ct,Ctr    : Word;
             OwlType   : Byte;
             MaleLoc   : Word;
             OffMap    : Boolean;
             NewX,NewY : Word;
             Owl       : Byte;
             Age1, Age2  : Byte;
          LABEL
               JmpOut,JmpOut2;
          BEGIN
               NewX := X;
               NewY := Y;
               GetSurroundings(X,Y,Types,BorderType);
               WITH Pop DO BEGIN
                    OwlType := GetOwl(Types[7]);
                    IF (OwlType >= SettledMale) THEN BEGIN
                       MaleLoc := FindPair(X,Y,Pop);
                       MakePair(X,Y,Pop,MaleLoc,Loc);
                       IF PairPop^[PairCtr].MaleAge <= PairPop^[PairCtr].FemaleAge THEN
                           Age2 := PairPop^[PairCtr].MaleAge
                       ELSE Age2 := PairPop^[PairCtr].FemaleAge;
                       IF GraphOn THEN BEGIN
                           GraphicChangeBird(X,Y);
                           GraphicChangeBird(NewX,NewY);
                       END;
                       Owl := SearchForOwls( X,Y,Pop, OwlPair, Age2);
                       IF (Owl = 0) THEN
                          NoMultiple(Habitat[Y]^[X]);

                    END
                    ELSE BEGIN
                         Ct := Succ(RandWord(6));
                         FOR Ctr := 1 TO 6 DO BEGIN
                               Ct := Succ(Ctr) MOD 6;
                               IF Ct = 0 THEN
                                  Ct := 6;
                               OwlType := GetOwl(Types[Ct]);
                               IF OwlType >= SettledMale THEN
                                  GOTO JmpOut;
                         END;
                         GOTO JmpOut2;
                         JmpOut:
                         IF  (Rand <= FemSearchProb)
                            THEN BEGIN
                            GetCoordinates(NewX,NewY,Ct,
                                           BorderType,OffMap);
                            MaleLoc := FindPair(NewX,NewY,Pop);
                            MoveOwl(X,Y,NewX,NewY,Loc,OwlDir,Pop,
                                          SFemale);
                         END
                         ELSE BEGIN
                              JmpOut2:
                              OwlDir := ChooseMove(Types,
                                                   GetDirection(Loc,Pop,
                                                   SFemale),SFemale);
                              IF OwlDir = 7 THEN
                                 NoMove(Loc,Pop,SFemale)
                              ELSE BEGIN
                                 GetCoordinates(NewX,NewY,OwlDir,BorderType,OffMap);
                                 IF OffMap THEN
                                    KillOwlOrPair(X,Y,Loc,Pop,SFemale)
                                 ELSE
                                     MoveOwl(X,Y,NewX,NewY,Loc,OwlDir,Pop,
                                             SFemale);
                              END;
                         END;
                    END;
               END;
END;
PROCEDURE MaleMove(    X,Y,Loc    : Word;
                   VAR Pop        : Population;
                       BorderType : Byte);
          VAR
             OwlDir    : Byte;
             Types     : LandTypeArr;
             OwlType   : Byte;
             OffMap    : Boolean;
             NewX,NewY : Word;
          BEGIN
               NewX := X;
               NewY := Y;
               WITH Pop DO BEGIN
                    GetSurroundings(X,Y,Types,BorderType);
                    OwlDir := ChooseMove(Types,GetDirection(Loc,Pop,SMale),
                                          SMale);
                    IF OwlDir = 7 THEN
                       NoMove(Loc,Pop,SMale)
                    ELSE BEGIN
                       GetCoordinates(NewX,NewY,OwlDir,BorderType,OffMap);
                       IF OffMap THEN BEGIN
                          KillOwlOrPair(X,Y,Loc,Pop,SMale)
                       END
                       ELSE
                          MoveOwl(X,Y,NewX,NewY,Loc,OwlDir,Pop,SMale);
                    END;
               END;
END;

Procedure ChangeLocation(x,y : Word;VAR MapLoc : Byte;VAR Pop : Population);
          VAR
             Owl           : Byte;
             SearchLoc     : Word;
             LandClass     : Single;
          begin
            with pop do begin
               Owl       := GetOwl(MapLoc);
               LandClass := LandClassOK[GetClass(MapLoc)];
               if (Owl > SFemale) and (LandClass <= rand) then begin
                  Case Owl of
                       SettledMale : begin
                                           AddOwl(SMale,MapLoc);
                                           SearchLoc :=
                                             Search(MaleCtr,x,y,
                                             SizeOf(SingleMale),
                                             MalePop^);
                                             if SearchLoc = 0 then begin
                                                WriteLn('Error');
                                                Halt;
                                             end;
                                             MalePop^[SearchLoc].Settled :=
                                                FALSE;
                                      end;
                         OwlPair  : begin
                                           SplitPair(X,Y,Pop);
                                      end;
               end;
          end;
       end;
end;
Procedure InspectMap(x,y : Word;VAR Map : HabitatArr;VAR Pop : Population);
          VAR
             Ct,Ct2 : Word;
          begin
               For Ct := 1 to y do begin
                   for Ct2 := 1 to x do begin
                       ChangeLocation(Ct2,Ct,Map[Ct]^[Ct2],Pop);
                   end;
               end;
end;
Procedure DoDyamicMap(    X,Y,Year : Word;
                      VAR Map      : HabitatArr;
                      VAR Pop      : Population;
                      VAR FileName : String);

          begin
             If ReadBoundStack(FileName,X,Y,Year,Map) then begin
                InspectMap(x,y,Map,Pop);
                if GraphOn then begin
                   DisplayArray(Habitat);
                end;
             end;
end;
BEGIN
END.

