UNIT Menu123;

INTERFACE

USES Crt,ScrLib,Utility;

        CONST
             Max_Comment_Ln = 50;    {make these as small as possible}
             Max_Menu       = 8;
             Max_Menu_St    = 10;
        TYPE
            CommentString = String[Max_Comment_Ln];
            MenuString    = String[Max_Menu_St];
            WordArr       = Array[1..80] OF Word;
            StrArr        = Array[1..Max_Menu] OF MenuString;
            MsgArr        = Array[1..Max_Menu] OF CommentString;
            PositionArr   = Array[1..Max_Menu] OF Byte;
            ColorSet123   = Array[1..5] OF Byte;
            Menu = RECORD
                 FirstLine   : StrArr;       {the top line}
                 SecondLine  : MsgArr;       {associated messages}
                 FstCharSet  : MenuString;   {all of the first letters}
                 Locations   : PositionArr;  {empty at initialization}
                 EntryNo     : Byte;         {number of choices on line 1}
                 Position    : Byte;         {where the bar starts}

                 {Att = background * 16 + forground}

                 TopLineAtt  : Byte;         {various colors}
                 BotLineAtt  : Byte;
                 FstCharAtt  : Byte;
                 LightBar    : Byte;
                 FirstBar    : Byte;
            END;

FUNCTION DoMenu123(VAR InputMenu : Menu;VAR Colors : ColorSet123) : Char;

IMPLEMENTATION
(***** Procedures begin here *****)

{GetKeys makes sure that extended keystrokes do not hang}

PROCEDURE GetKeys(VAR Ch,EscCh : Char);
           BEGIN
                EscCh := #0;
                Ch := UpCase(ReadKey);
                IF Ch = #0 THEN
                   EscCh := ReadKey;
END;

{ClrLine uses CharToScreen to fill a line with blanks}

PROCEDURE ClrLine(LineNo,Att : Byte);
          BEGIN
               CharToScreen(Pred(LineNo) * 160,80,#32,Att);
END;

{HighLightWrite Writes a string and highlights the first letter}

PROCEDURE HighLightWrite(Posit : Word;Att,HighAtt : Byte; VAR St : String);
          BEGIN
               FastWrite(Posit,Att,St);
               SetAttr(Posit,1,HighAtt);
END;

{BarAttribute writes a highlighted light bar}

PROCEDURE BarAttribute(Posit,Length : Word;FirstAtt, Att : Byte);
          BEGIN
               SetAttr(Posit,Length,Att);
               SetAttr(Posit,1,FirstAtt);
END;

{WriteFirstLine spaces the first line elements evenly across the line}

PROCEDURE WriteFirstLine(VAR Line             : StrArr;
                             StrNo            : Byte;
                         VAR StartPos         : PositionArr;
                             FirstAtt,Att     : Byte);
          VAR
             Ct,Jump,
             Temp    : Byte;
          BEGIN
               ClrLine(1,Att);                        {blank the line}
               Temp := (80 DIV StrNo) * 2;            {get the skip}
               Jump := 0;
               FOR Ct := 1 TO StrNo DO BEGIN       {write the rest}
                   StartPos[Ct] := Jump;
                   HighlightWrite(Jump,Att,FirstAtt,Line[Ct]);
                   Inc(Jump,Temp);
               END;
END;

{WriteSecondLine clears the second line and writes the message string.}

PROCEDURE WriteSecondLine(VAR Line : MsgArr;Posit,Att : Byte);
          BEGIN
               ClrLine(2,Att);
               FastWrite(160,Att,Line[Posit]);
END;

{MoveLightBar moves the light bar around and checks for appropriate keystrokes}

PROCEDURE MoveLightBar(VAR InputMenu                    : Menu;
                       VAR LetterCh                     : Char);

     VAR
        Ch,EscCh : Char;
     BEGIN
          WITH InputMenu DO BEGIN

               {turn bar on at position Position}

               BarAttribute(Locations[Position],Length(FirstLine[Position]),
                            FirstBar,LightBar);

               {Write the associated message on line2}

               WriteSecondLine(SecondLine,Position,BotLineAtt);

               WHILE TRUE DO BEGIN                          {endless loop}

                     {halt until an appropriate key is hit}

                     REPEAT
                           GetKeys(Ch,EscCh);
                     UNTIL ((CharPos(Ch,FstCharSet) <> 0)
                           OR (EscCh IN [#75,#77])
                           OR (Ch IN [#13,#27]));

                     {turn off bar}

                     BarAttribute(Locations[Position],
                                  Length(FirstLine[Position]),
                                  FstCharAtt,TopLineAtt);

                     {take appropriate action based on the keystroke}

                     CASE Ch OF
                          #0 : BEGIN
                                   CASE EscCh OF
                                        #75 : BEGIN
                                                   Dec(Position);
                                                   IF Position = 0 THEN
                                                      Position := EntryNo;
{move the Light bar}                          END;
                                        #77 : BEGIN
                                                   Inc(Position);
                                                   IF Position > EntryNo THEN
                                                      Position := 1;
                                           END;
                                   END; {case}
                               END;
{Enter hit}             #13 :  BEGIN
                                    LetterCh := FstCharSet[Position];
                                    Exit;
                               END;
{Esc. hit}              #27 :  BEGIN
                                    LetterCh := #27;
                                    Exit;
                               END;
{a first letter hit}     ELSE  BEGIN
                                  LetterCh := Ch;
{move light bar position}         Position := CharPos(Ch,FstCharSet);
                                  Exit;
                               END;
                      END;     {case}

                 {turn the bar back on}

                      BarAttribute(Locations[Position],
                                   Length(FirstLine[Position]),
                                   FirstBar,LightBar);

                 {and write the appropriate message}

                      WriteSecondLine(SecondLine,Position,BotLineAtt);
               END;
          END;
END;

{change colors if it is a color monitor}

PROCEDURE ColorChange123(VAR InputMenu : Menu;VAR Colors : ColorSet123);
          BEGIN
               IF Color THEN BEGIN
                  WITH InputMenu DO BEGIN
                       TopLineAtt := Colors[1];
                       BotLineAtt := Colors[2];
                       FstCharAtt := Colors[3];
                       LightBar   := Colors[4];
                       FirstBar   := Colors[5];
                  END;
               END;
END;
{the whole process is called very simply}

FUNCTION DoMenu123(VAR InputMenu : Menu;VAR Colors : ColorSet123) : Char;
          VAR
             OutChar : Char;
          BEGIN
               ColorChange123(InputMenu,Colors);
               WITH InputMenu DO BEGIN
                    WriteFirstLine(FirstLine,EntryNo,
                                   Locations,FstCharAtt,
                                   TopLineAtt);
                    MoveLightBar(InputMenu,OutChar);
               END;
               DoMenu123 := OutChar;

END;
BEGIN
END.










