

UNIT OwlGraph;
INTERFACE

USES Global,Graph,OwlAsm,Math,Crt,VGAPal,ClassDef;

PROCEDURE GraphicDisplay2(FirstTime : Boolean);
PROCEDURE MOGraphicDisplay(FirstTime : Boolean;D,M : Integer);
PROCEDURE GraphicChangeBird(X,Y : Integer);
PROCEDURE InterceptKeys(VAR EscSw : Boolean);
PROCEDURE MOInterceptKeys(VAR EscSw : Boolean);
PROCEDURE InitializeGraphics;
PROCEDURE RestoreTextMode;
PROCEDURE RestoreGraphics;
PROCEDURE OutputStats(Yr,Pairs,Juveniles,Adults : Word);
PROCEDURE WriteMessage(StartX : Word;Message : String);
PROCEDURE DisplayArray(VAR Arr : HabitatArr);

CONST
     Device  : Integer       = 0;
     Mode    : Integer       = 0;

IMPLEMENTATION
TYPE
    MOArray      = Array[1..10] OF Byte;
    MoStringArr  = Array[1..10] OF String[20];
CONST
     OwlTypeLetter : Array[0..4] OF Char = (' ','M','F','P');
     AltOwlTypeLetter : Array[0..8] OF Char = (' ','*','*','*');
     OwlTypeColor  : Array[0..4] OF Byte = (0,White,LightRed,
                                            LightMagenta,Yellow);
     OwlTypeName   : Array[1..3] OF String[10] =
     ('Male', 'Female', 'Pair');
     MOColor : MOArray = (Black,Blue,LightBlue,Cyan,
                          Green,LightGreen,Yellow,
                          Red,LightRed,LightMagenta);
     MOName  : MOStringArr = ('0-10%','10-20%','20-30%','30-40%','40-50%',
                              '50-60%','60-70%','70-80%','80-90%','90-100%');


     BlockOn    : Boolean    = FALSE;
     OwlOn      : Boolean    = FALSE;
     MoveFlag   : Boolean    = FALSE;
TYPE
    LandscapeClass = Array[1..LandscapeClasses] OF Integer;
    BitMap         = Array[1..10000] OF Byte;
    String4        = String[4];
VAR
   HexagonArr           : Array[1..7,1..2] OF Integer;
   HexSin30,HexCos30,
   HexRad               : Single;

PROCEDURE InitializeGraphics;
          BEGIN
               InitGraph(Device,Mode,'');
               IF GraphResult <> 0 THEN
                  FatalErrors(1);
               GraphOn := TRUE;
END;
PROCEDURE RestoreTextMode;
          BEGIN
               ClearDevice;
               GraphOn := FALSE;
               RestoreCrtMode;
END;
PROCEDURE RestoreGraphics;
          BEGIN
               SetGraphMode(Mode);
               GraphOn := TRUE;
END;

(****Graphics I/O****)

{HercChange sets map to hercules standards in scale and color}

PROCEDURE HercChange;
          BEGIN
               ClassArray   := HercClassArray;
               ClassPattern := HercClassPattern;
               HexCos30 := HexCos30 * 1.6;
END;
PROCEDURE EraseArea(X1,Y1,X2,Y2:Word);
          BEGIN
               SetFillStyle(0,0);
               Bar(X1,Y1,X2,Y2);
END;

PROCEDURE GraphErrors(Low,High : Integer;ErrNo : Byte;VAR EscSw : Boolean);
          VAR
             Temp1,Temp2       : String[10];
             StartX,StartY     : Word;
             Ch,Ch2            : Char;
          BEGIN
               StartX := 140;
               StartY := 110;
               SetTextJustify(1,0);
               CASE ErrNo OF
                   1 : OutTextXY(StartX,StartY,'Bad numeric format');
                   2 : BEGIN
                           Str(Low,Temp1);
                           Str(High,Temp2);
                           OutTextXY(StartX,StartY,'Value out of bounds');
                           OutTextXY(StartX,StartY+10,Temp1 + '  X  '+ Temp2);
                        END;
                   3 : OutTextXY(StartX,StartY,'Invalid coordinates');
               END;
               OutTextXY(StartX,StartY+20,'<ESC> to quit');
               GetKeys(Ch,Ch2);
               EscSw := (Ch = #27);
               EraseArea(55,100,225,130);
END;
PROCEDURE SetGraphCursor(Y : Word);
          BEGIN
               OutTextXY(GetX,Y,'_');
END;
PROCEDURE KillGraphCursor(Y : Word);
          BEGIN
               SetColor(Black);
               OutTextXY(GetX,Y,'_');
               SetColor(White);
END;
FUNCTION InputGraphicInteger(     X,Y,Field : Word;
                                  Low,High  : Integer;
                              VAR Value     : Integer) : Boolean;
         VAR
              St          : String[20];
              Ch,Ch2      : Char;
              Err         : Integer;
              Holder      : Integer;
              TempX,TempY : Word;
              EscSw       : Boolean;
         LABEL
              LOOP;
         BEGIN
              TempX := x;
              TempY := y;
              Holder := Value;
              LOOP:
              SetTextJustify(0,0);
              X := TempX;
              Y := TempY;
              EraseArea(x,y-8,x+8*Field,y);
              St[0] := #0;
              MoveTo(x,y);
              REPEAT
                    SetGraphCursor(y);
                    GetKeys(Ch,Ch2);
                    KillGraphCursor(y);
                    IF (Ch IN ['0'..'9','-']) AND
                       (St[0] < Char(Field)) THEN BEGIN
                       St := St + Ch;
                       OutText(Ch);
                       SetGraphCursor(Y);
                    END;
                    IF (Ch = #8) AND (St[0] <> #0) THEN BEGIN
                          Dec(St[0]);
                          MoveTo(GetX-8,y);
                          Bar(GetX,y-8,GetX+8,y);
                          SetGraphCursor(Y);
                    END;
                    IF Ch = #27 THEN BEGIN
                       InputGraphicInteger := TRUE;
                       Exit;
                    END;
              UNTIL Ch = #13;
              IF St[0] = #0 THEN BEGIN
                  KillGraphCursor(Y);
                  Str(Value:Field,St);
                  OutText(St);
                  InputGraphicInteger := FALSE;
              END
              ELSE BEGIN
                   Val(St,Holder,Err);
                   IF Err <> 0 THEN BEGIN
                      GraphErrors(Low,High,1,EscSw);
                      IF EscSw THEN BEGIN
                         InputGraphicInteger := TRUE;
                         Exit;
                      END
                      ELSE
                          GOTO LOOP;
                   END;
                   IF (Holder < Low) OR (Holder > High) THEN BEGIN
                      GraphErrors(Low,High,2,EscSw);
                      IF EscSw THEN BEGIN
                         InputGraphicInteger := TRUE;
                         Exit;
                      END
                      ELSE
                          GOTO LOOP;
                   END;
                   Value        := Holder;
                   InputGraphicInteger := FALSE;
             END;
             KillGraphCursor(Y);
END;
PROCEDURE OpenWindow(StartX,StartY,EndX,EndY : Word; VAR Area : Bitmap);
          BEGIN
               IF ImageSize(StartX,StartY,EndX,EndY) >
                  SizeOf(Area) THEN BEGIN
                  CloseGraph;
                  HaltProgram;
               END;
               GetImage(StartX,StartY,EndX,EndY,Area);
               SetFillStyle(0,0);
               Bar3d(StartX,StartY,EndX,EndY,0,TRUE);
END;
PROCEDURE ZoomMenu(VAR EscSw : Boolean);
          VAR
             z                    : BitMap;
             StartX,StartY        : Word;
             NessMem              : Word;
          LABEL
               LOOP;
          BEGIN
               SetColor(White);
               LOOP:
               StartX := 50;
               StartY := 50;
               SetTextJustify(0,0);
               OpenWindow(StartX,StartY,StartX + 178,StartY + 90,z);
               Inc(StartY,10);
               Inc(StartX,5);
               OutTextXY(StartX,StartY,'Start column :');
               OutTextXY(StartX,StartY+10,   'Start row    :');
               OutTextXY(StartX,StartY+20,'Stop  column :');
               OutTextXY(StartX,StartY+30,'Stop  row    :');
               StartX := 190;
               EscSw := InputGraphicInteger(StartX,StartY,3,1,
                                            HabitatWidth-4,ZoomXStart);
                IF EscSw THEN BEGIN
                   PutImage(50,50,z,CopyPut);
                   Exit;
               END;
               EscSw := InputGraphicInteger(StartX,StartY+10,3,1,
                                            HabitatLength-4,ZoomYStart);
                IF EscSw THEN BEGIN
                   PutImage(50,50,z,CopyPut);
                   Exit;
                END;
                EscSw := InputGraphicInteger(StartX,StartY+20,3,ZoomXStart+4,
                                            HabitatWidth,ZoomXStop);
                IF EscSw THEN BEGIN
                   PutImage(50,50,z,CopyPut);
                   Exit;
                END;
                EscSw := InputGraphicInteger(StartX,StartY+30,3,ZoomYStart+4,
                                            HabitatLength,ZoomYStop);
                IF EscSw THEN BEGIN

                   Exit;
                END;
                IF (ZoomXStart + 4 > ZoomXStop) OR (ZoomYStart + 4 > ZoomYStop) THEN
                   BEGIN
                        GraphErrors(1,1,3,EscSw);
                        IF EscSw THEN
                           PutImage(50,50,z,CopyPut)
                        ELSE
                            GOTO LOOP;
                END;

END;

(****Graphics procedures associated with display of the habitat array****)

PROCEDURE DrawHexagon(StartX,StartY : Single;Class : Word);
          VAR
             Owl : Byte;
          BEGIN
               SetColor(ExteriorColor);
               HexagonArr[1,1] := Round(StartX + HexCos30);
               HexagonArr[1,2] := Round(StartY + HexSin30);
               HexagonArr[2,1] := HexagonArr[1,1];
               HexagonArr[2,2] := Round(StartY - HexSin30);
               HexagonArr[3,1] := Round(StartX);
               HexagonArr[3,2] := Round(StartY - HexRad);
               HexagonArr[4,1] := Round(StartX - HexCos30);
               HexagonArr[4,2] := HexagonArr[2,2];
               HexagonArr[5,1] := HexagonArr[4,1];
               HexagonArr[5,2] := HexagonArr[1,2];
               HexagonArr[6,1] := Round(StartX);
               HexagonArr[6,2] := Round(StartY + HexRad);
               HexagonArr[6,1] := Round(StartX);
               HexagonArr[6,2] := Round(StartY + HexRad);
               SetFillStyle(ClassPattern[GetClass(Class)],
                            ClassArray[GetClass(Class)]);
               FillPoly(6,HexagonArr);
               Owl := GetOwl(Class);
               IF Owl <> 0 THEN BEGIN
                  SetColor(OwlTypeColor[Owl]);
                  SetTextJustify(1,1);
                  IF HexRad < 8 THEN
                  OutTextXY(Round(StartX),Round(StartY+1),
                                  AltOwlTypeLetter[Owl])
                  ELSE
                     OutTextXY(Round(StartX),Round(StartY+1),
                                     OwlTypeLetter[Owl]);
                  SetColor(ExteriorColor);
               END;
END;
PROCEDURE DrawMOHexagon(StartX,StartY : Single;Class : Word);
          VAR
             Owl : Byte;
          BEGIN
               SetColor(ExteriorColor);
               HexagonArr[1,1] := Round(StartX + HexCos30);
               HexagonArr[1,2] := Round(StartY + HexSin30);
               HexagonArr[2,1] := HexagonArr[1,1];
               HexagonArr[2,2] := Round(StartY - HexSin30);
               HexagonArr[3,1] := Round(StartX);
               HexagonArr[3,2] := Round(StartY - HexRad);
               HexagonArr[4,1] := Round(StartX - HexCos30);
               HexagonArr[4,2] := HexagonArr[2,2];
               HexagonArr[5,1] := HexagonArr[4,1];
               HexagonArr[5,2] := HexagonArr[1,2];
               HexagonArr[6,1] := Round(StartX);
               HexagonArr[6,2] := Round(StartY + HexRad);
               HexagonArr[6,1] := Round(StartX);
               HexagonArr[6,2] := Round(StartY + HexRad);
               if Device <> 9 then
                  SetFillStyle(1,MOColor[GetOwl(Class)])
               else
                   SetFillStyle(1,Pred(GetOwl(Class)));
               FillPoly(6,HexagonArr);
               SetColor(ExteriorColor);
END;
PROCEDURE FindParams(VAR Width,Radius : Single);
          VAR
             MaxX,MaxY                 : Word;
             ScaleX                    : Single;
             Scalar,DimX,DimY          : Word;
          BEGIN
               DimX := Succ(ZoomXStop-ZoomXStart);
               DimY := Succ(ZoomYStop-ZoomYStart);
               IF DimX > DimY THEN
                  Scalar := DimX
               ELSE
                   Scalar := DimY;
               MaxX := GetMaxX;
               MaxY := GetMaxY;
               IF Scalar > 10 THEN
                  ScaleX := 0.58 * MaxY/MaxX
               ELSE
                   ScaleX := 0.50 * MaxY/MaxX;
               Radius := (MaxX * ScaleX)/Scalar;
               HexSin30 := Sin(30 * DegToRad) * Radius;
               HexCos30 := Cos(30 * DegToRad) * Radius;
               Width := HexCos30;
               HexRad := Radius;
END;
PROCEDURE WriteDimensions;
          VAR
             St1,St2    : String[4];
             Xloc,YLoc  : Word;
          BEGIN
               SetTextJustify(1,1);
               XLoc := GetMaxX - 70;
               YLoc := GetMaxY - 50;
               Rectangle(XLoc - 40,Yloc,XLoc+40,YLoc + 45);
               Str(ZoomXStart,St1);
               Str(ZoomYStart,St2);
               OutTextXY(XLoc,YLoc+12,'('+ St1 + ',' + St2+ ')');
               OutTextXY(XLoc,YLoc+24,'TO');
               Str(ZoomXStop,St1);
               Str(ZoomYStop,St2);
               OutTextXY(XLoc,YLoc+36,'('+ St1 + ','+ St2 + ')');
END;
PROCEDURE DisplayArray(VAR Arr : HabitatArr);
          VAR
             Ct1,Ct2                      : Word;
             Ch,ExtCh                     : Char;
             Radius,StartX,
             StartY,Width                 : Single;
          BEGIN
               SetColor(ExteriorColor);
               FindParams(Width,Radius);
               IF Device = HercMono THEN
               HercChange;
               Width := HexCos30;
               StartY := 1;
               FOR Ct1 := ZoomYStart TO ZoomYStop DO BEGIN
                   StartY := StartY + 1.5 * Radius;
                   StartX := 1;
                   IF Odd(Ct1) THEN
                          StartX := StartX + Width;
                   FOR Ct2 := ZoomXStart TO ZoomXStop DO BEGIN
                       StartX := StartX + 2 * Width;
                       DrawHexagon(StartX,StartY,Arr[Ct1]^[Ct2]);
                   END;
               END;
               WriteDimensions;
               Rectangle(1,GetMaxY - 25,GetMaxX - 110,GetMaxY-5);
               Line(GetMaxX-110,1,GetMaxX-110,GetMaxY-50);
END;
PROCEDURE DisplayMOArray(VAR Arr : HabitatArr);
          VAR
             Ct1,Ct2                      : Word;
             Ch,ExtCh                     : Char;
             Radius,StartX,
             StartY,Width                 : Single;
          BEGIN
               SetColor(ExteriorColor);
               FindParams(Width,Radius);
               IF Device = HercMono THEN
               HercChange;
               Width := HexCos30;
               StartY := 1;
               FOR Ct1 := ZoomYStart TO ZoomYStop DO BEGIN
                   StartY := StartY + 1.5 * Radius;
                   StartX := 1;
                   IF Odd(Ct1) THEN
                          StartX := StartX + Width;
                   FOR Ct2 := ZoomXStart TO ZoomXStop DO BEGIN
                       StartX := StartX + 2 * Width;
                       DrawMOHexagon(StartX,StartY,Arr[Ct1]^[Ct2]);
                   END;
                   IF KeyPressed THEN BEGIN
                      GetKeys(Ch,ExtCh);
                      IF Ch = #27 THEN
                         HaltProgram;
                   END;
               END;
               WriteDimensions;
               Rectangle(1,GetMaxY - 25,GetMaxX - 110,GetMaxY-5);
               Line(GetMaxX-110,1,GetMaxX-110,GetMaxY-50);
END;

(****Legend is used for exclusively for modifying land classification****)

PROCEDURE Legend;
          VAR
             Pos1,Pos2,YPos,
             YLetter        : Word;
             Ct             : Byte;
             IncY,DistY     : Word;
          BEGIN
               SetColor(White);
               SetTextJustify(1,1);
               Pos1 := GetMaxX - 60;
               Pos2 := Pos1 - 26;
               Setcolor(LightGray);
               OutTextXY(Pos1,10,'LAND CLASS');
               SetColor(White);
               YPos := 0;
               IF Device <> 9 THEN BEGIN
                  IncY := 20;
                  DistY := 10;
               END
               ELSE BEGIN
                   IncY := 30;
                   DistY := 20;
               END;
               FOR Ct := 1 TO LandscapeClasses DO BEGIN
                   Inc(YPos,IncY);
                   OutTextXY(Pos1,Ypos,ClassNameArray[Ct]);
                   SetFillStyle(ClassPattern[Ct],ClassArray[Ct]);
                   Bar3d(Pos2,YPos+5,Pos2+50,YPos + DistY,0,TRUE);
               END;
               Inc(YPos,IncY + 5);
               SetColor(LightCyan);
               Line(GetMaxX - 110,Ypos,GetMaxX-30,Ypos);
               SetColor(LightGray);
               Inc(YPos,15);
               OutTextXY(Pos1,Ypos,'OWLS');
               Setcolor(White);
               Dec(YPos,10);
               YLetter := 13;
               IF Device <> 9 THEN BEGIN
                  IncY    := 22;
                  DistY   := 13;
                  YLetter := 11;
               END;
               FOR Ct := 1 TO OwlClasses DO BEGIN
                   Inc(YPos,IncY);
                   OutTextXY(Pos1,Ypos,OwlTypeName[Ct]);
                   SetFillStyle(1,Black);
                   Bar3d(Pos2,YPos+5,Pos2+50,YPos + DistY,0,TRUE);
                   SetColor(OwlTypeColor[Ct]);
                   OutTextXY(Pos1,YPos + YLetter,OwlTypeLetter[Ct]);
                   SetColor(White);
               END;
END;
PROCEDURE MOLegend;
          VAR
             Pos1,Pos2,YPos,
             YLetter        : Word;
             Ct             : Byte;
             IncY,DistY     : Word;
          BEGIN
               SetColor(White);
               SetTextJustify(1,1);
               Pos1 := GetMaxX - 60;
               Pos2 := Pos1 - 26;
               Setcolor(LightCyan);
               OutTextXY(Pos1,10,'MEAN OCC.');
               SetColor(White);
               YPos := 0;
               IF Device <> 9 THEN BEGIN
                  IncY := 20;
                  DistY := 10;
               END
               ELSE BEGIN
                   IncY := 30;
                   DistY := 20;
               END;
               FOR Ct := 1 TO 10 DO BEGIN
                   Inc(YPos,IncY);
                   OutTextXY(Pos1,Ypos,MOName[Ct]);
                   if Device <> 9 then
                      SetFillStyle(1,MOColor[Ct])
                   else
                       SetFillStyle(1,Pred(Ct));
                   Bar3d(Pos2,YPos+5,Pos2+50,YPos + DistY,0,TRUE);
               END;
END;
PROCEDURE WriteMessage(StartX : Word;Message : String);
          VAR
             St : String[5];
          BEGIN
               EraseArea(2,GetMaxY - 24,GetMaxX - 111,GetMaxY-6);
               SetColor(White);
               OutTextXY(StartX,GetMaxY - 10,Message);
               SetColor(LightBlue);
               Str(GlobalDelay DIV 10:2,St);
               OutTextXY(10,GetMaxY - 10,St);
               SetColor(White);
END;
PROCEDURE GraphicDisplay2(FirstTime : Boolean);
          BEGIN
               IF FirstTime THEN
                  InitializeGraphics
               ELSE
                   SetGraphMode(Mode);
               DisplayArray(Habitat);
               Legend;
               SetTextJustify(0,0);

END;
PROCEDURE MOGraphicDisplay(FirstTime : Boolean;D,M : Integer);
          BEGIN
               IF FirstTime THEN
                  InitializeGraphics
               ELSE BEGIN
                    Mode   := m;
                    Device := d;
                   SetGraphMode(Mode);
               END;
               if Device = 9 then
                  ChangePalette(PaletteNo);
               DisplayMOArray(Habitat);
               MOLegend;
               SetTextJustify(0,0);
END;
(****These procedures affect bird movement on screen*****)

FUNCTION ComputePosX(X,Y : Word) : Word;
         BEGIN
              IF Odd(ZoomYStart) THEN BEGIN
                 IF Odd(Y) THEN
                    ComputePosX := 1 + Round(HexCos30)
                                     + Round(HexCos30 * 2 * x)
                 ELSE
                     ComputePosX  := 1 + Round(HexCos30 * 2 * x);
              END
              ELSE BEGIN
                   IF NOT Odd(Y) THEN
                    ComputePosX := 1 + Round(HexCos30)
                                     + Round(HexCos30 * 2 * x)
                 ELSE
                     ComputePosX  := 1 + Round(HexCos30 * 2 * x);
              END;
END;
FUNCTION ComputePosY(Y : Word) : Word;
         BEGIN
             ComputePosY := Round(HexRad * 1.5 * Y) + 2;
END;
PROCEDURE EraseLetter(X,Y,MapX,MapY: Word);
          VAR
             Class     : Byte;
             PosX,PosY : Word;
          BEGIN
               Class := GetClass(Habitat[Y]^[X]);
               SetFillStyle(ClassPattern[Class],ClassArray[Class]);
               PosX := ComputePosX(MapX,MapY);
               PosY := ComputePosY(MapY);
               Bar(PosX-5,PosY-5,PosX+5,PosY+5);
END;
PROCEDURE WriteLetter(X,Y,MapX,MapY: Word);
          VAR
             Class     : Byte;
             PosX,PosY : Word;
          BEGIN
               Class := GetOwl(Habitat[Y]^[X]);
               IF Class <> 0 THEN BEGIN
                  SetColor(OwlTypeColor[Class]);
                  PosX := ComputePosX(MapX,MapY);
                  PosY := ComputePosY(MapY);
                  SetTextJustify(1,1);
                  OutTextXY(PosX,PosY,OwlTypeLetter[Class]);
               END;

END;
FUNCTION OnMap(X,Y : Integer) : Boolean;
          BEGIN
               OnMap := (((X >= ZoomXStart) AND (X <= ZoomXStop)) AND
                         ((Y >= ZoomYStart) AND (Y <= ZoomYStop)));
END;
PROCEDURE GraphicChangeBird(X,Y : Integer);
          VAR
             MapY,MapX : Integer;
             PosX,PosY : Word;
          BEGIN
               IF OnMap(X,Y) THEN BEGIN
                  MapX := X - Pred(ZoomXStart);
                  MapY := Y - Pred(ZoomYStart);
                  IF HexRad < 10 THEN BEGIN
                     PosX := ComputePosX(MapX,MapY);
                     PosY := ComputePosY(MapY);
                     Dec(PosY,2);
                     DrawHexagon(PosX,PosY,Habitat[Y]^[X])
                  END
                  ELSE BEGIN
                       EraseLetter(X,Y,MapX,MapY);
                       WriteLetter(X,Y,MapX,MapY);
                  END;
               END;
END;
PROCEDURE InterceptKeys(VAR EscSw : Boolean);
          VAR
             Ch,Ch2 : Char;
             EscSw2 : Boolean;
          LABEL
               JMPOUT;
          BEGIN
               EscSw := FALSE;
               IF KeyPressed THEN BEGIN
                  SetTextJustify(0,0);
                  GetKeys(Ch,Ch2);
                  WriteMessage(80,
                   'Any key to continue, <F1> to zoom, <ESC> to quit');
                  REPEAT
                       IF Ch = #27 THEN BEGIN
                          EscSw := TRUE;
                          Exit;
                       END;
                       CASE Ch2 OF
                           #59 : BEGIN
                                      ZoomMenu(EscSw2);
                                      IF NOT EscSw2 THEN BEGIN
                                         GraphicDisplay2(FALSE);
                                         WriteMessage(80,
                           'Any key to continue, <F1> to zoom, <ESC> to quit');
                                      END;
                                 END;
                           #80 : BEGIN
                                      IF GlobalDelay < 1000 THEN
                                         Inc(GlobalDelay,10);
                                         GOTO JMPOUT;
                                 END;
                           #72 : BEGIN
                                      IF GlobalDelay >= 10 THEN
                                         Dec(GlobalDelay,10);
                                         GOTO JMPOUT;
                                 END;
                       END;
                       GetKeys(Ch,Ch2);
                  UNTIL (Ch2 <> #59) AND (Ch <> #27);
               JMPOUT:
                       WriteMessage(100,
                              'Any key to halt, <F1> to zoom, <ESC> to quit');
               END;
END;
Procedure WriteMOText;
          begin
               SetColor(White);
               if Device = 9 then
               OutTextXY(40,GetMaxY - 10,'<F1> to zoom, <F2> to '+
                            'change palette, <ESC> to quit')
               Else
               OutTextXY(180,GetMaxY - 10,'<F1> to zoom, '+
                                          '<ESC> to quit');
end;
PROCEDURE MOInterceptKeys(VAR EscSw : Boolean);
          VAR
             Ch,Ch2 : Char;
             EscSw2 : Boolean;
          BEGIN
               EscSw := FALSE;
               SetTextJustify(0,0);
                WriteMOText;
                  REPEAT
                       GetKeys(Ch,Ch2);
                       IF Ch = #27 THEN BEGIN
                          EscSw := TRUE;
                          Exit;
                       END;
                       CASE Ch2 OF
                           #59 : BEGIN
                                      ZoomMenu(EscSw2);
                                      IF NOT EscSw2 THEN BEGIN
                                         MOGraphicDisplay(FALSE,Device,Mode);
                                         WriteMOText;
                                      END;
                                  END;
                            #60 : if Device = 9 then NewPalette;
                       END;
                  UNTIL Ch = #27;
END;

PROCEDURE OutputStats(Yr,Pairs,Juveniles,Adults : Word);
          VAR
             St : String;
          BEGIN
               SetTextJustify(0,0);
               IF Device <> HercMono THEN
                  SetFillStyle(1,Red)
               ELSE
                   SetFillStyle(1,0);
               BAR(1,GetMaxY - 40,GetMaxX - 111,GetMaxY-26);
               SetColor(White);
               MoveTo(50,GetMaxY-28);
               Str(Yr:4,St);
               OutText('Year = ');
               OutText(St);
               Str(Pairs:4,St);
               OutText(' Pairs = ');
               OutText(St);
               Str(Juveniles:4,St);
               OutText(' Juv. = ');
               OutText(St);
               Str(Adults:4,St);
               OutText(' Adults = ');
               OutText(St);
END;

BEGIN
END.


